@@ -36,6 +36,7 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
     QUIT_ON_CLOSE = True
 
     MAX_DISPLAY = 24*60*60
+    TITLE_TEMPLATE = 'MercuryiTC Control'
 
     def __init__(self, mercury):
         super(self.__class__, self).__init__()
@@ -45,7 +46,7 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         self.feed = MercuryFeed(mercury)
 
         # sent Title font size relative to the system's default size
-        scaling = 1.8
+        scaling = 1.5
         font = self.labelTitle.font()
         defaultFontSize = QtWidgets.QLabel('test').font().pointSize()
         fontSize = round(defaultFontSize*scaling, 1)
@@ -89,13 +90,46 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
 
         # restore previous window geometry
         self.restore_geometry()
-        # Connect menu bar actions
-        self.set_up_menubar()
+
+        # connect to callbacks
+        self.modulesAction.triggered.connect(self.on_module_selection_clicked)
+        self.showLogAction.triggered.connect(self.on_log_clicked)
+        self.exitAction.triggered.connect(self.exit_)
+        self.readingsAction.triggered.connect(self.on_readings_clicked)
+        self.connectAction.triggered.connect(self.feed.connect)
+        self.disconnectAction.triggered.connect(self.feed.disconnect)
+        self.updateAddressAction.triggered.connect(self.connectionDialog.open)
+
+        self.t2_edit.returnPressed.connect(self.change_t_setpoint)
+        self.r1_edit.returnPressed.connect(self.change_ramp)
+        self.r2_checkbox.clicked.connect(self.change_ramp_auto)
+        self.gf1_edit.returnPressed.connect(self.change_flow)
+        self.gf2_checkbox.clicked.connect(self.change_flow_auto)
+        self.gf3_edit.returnPressed.connect(self.change_flow_min)
+        self.h1_edit.returnPressed.connect(self.change_heater)
+        self.h2_checkbox.clicked.connect(self.change_heater_auto)
+
+        # initially disable menu bar items, will be enabled later individually
+        self.connectAction.setEnabled(True)
+        self.disconnectAction.setEnabled(False)
+        self.modulesAction.setEnabled(False)
+        self.readingsAction.setEnabled(False)
+
+        self.build_sensor_menu()
+
+        # initially disable controls, will be enabled later individually
+        self.t2_edit.setEnabled(False)
+        self.r1_edit.setEnabled(False)
+        self.r2_checkbox.setEnabled(False)
+        self.gf1_edit.setEnabled(False)
+        self.gf2_checkbox.setEnabled(False)
+        self.h1_edit.setEnabled(False)
+        self.h2_checkbox.setEnabled(False)
 
         # check if mercury is connected, connect slots
         self.display_message('Looking for temperature controller at %s...' %
                              self.mercury.visa_address)
-        self.update_gui_connection(self.mercury.connected)
+        self.update_gui_connection(self.feed.connected)
 
         # start (stop) updates of GUI when mercury is connected (disconnected)
         # adjust clickable buttons upon connect / disconnect
@@ -108,6 +142,8 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         # set up logging to file
         self.setup_logging()
 
+        self.feed.connect()
+
 # =================== BASIC UI SETUP ==========================================
 
     def restore_geometry(self):
@@ -136,25 +172,6 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         else:
             self.hide()
 
-    def set_up_menubar(self):
-        """
-        Connects menu bar items to callbacks, sets their initial activation.
-        """
-        # connect to callbacks
-        self.modulesAction.triggered.connect(self.on_module_selection_clicked)
-        self.showLogAction.triggered.connect(self.on_log_clicked)
-        self.exitAction.triggered.connect(self.exit_)
-        self.readingsAction.triggered.connect(self.on_readings_clicked)
-        self.connectAction.triggered.connect(self.feed.connect)
-        self.disconnectAction.triggered.connect(self.feed.disconnect)
-        self.updateAddressAction.triggered.connect(self.connectionDialog.open)
-
-        # initially disable menu bar items, will be enabled later individually
-        self.connectAction.setEnabled(True)
-        self.disconnectAction.setEnabled(False)
-        self.modulesAction.setEnabled(False)
-        self.readingsAction.setEnabled(False)
-
     def on_slider_changed(self):
         # determine first plotted data point
         sv = self.horizontalSlider.value()
@@ -164,8 +181,24 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         self.canvas.p0.setXRange(-sv, 0)
         self.canvas.p0.enableAutoRange(x=False, y=True)
 
+    def build_sensor_menu(self):
+        self.sensorMenu = QtWidgets.QMenu()
+        self.sensorActionGroup = QtWidgets.QActionGroup(self)
+
+        for nick in self._get_nicks(MercuryITC_TEMP):
+            action = QtWidgets.QAction(nick)
+            action.setData(nick)
+            action.setCheckable(True)
+            action.setChecked(nick == self.feed.temperature_module_nick)
+            self.sensorMenu.addAction(action)
+            self.sensorActionGroup.addAction(action)
+            self.sensorActionGroup.triggered.connect(self.on_sensor_selected)
+
+        self.sensorAction.setMenu(self.sensorMenu)
+
     @QtCore.pyqtSlot(bool)
     def update_gui_connection(self, connected):
+
         if connected:
             self.display_message('Connection established.')
             self.led.setChecked(True)
@@ -175,15 +208,17 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
             self.disconnectAction.setEnabled(True)
             self.modulesAction.setEnabled(True)
             self.readingsAction.setEnabled(True)
+            self.sensorAction.setEnabled(True)
 
-            # connect user input to change mercury settings
-            self.t2_edit.returnPressed.connect(self.change_t_setpoint)
-            self.r1_edit.returnPressed.connect(self.change_ramp)
-            self.r2_checkbox.clicked.connect(self.change_ramp_auto)
-            self.gf1_edit.returnPressed.connect(self.change_flow)
-            self.gf2_checkbox.clicked.connect(self.change_flow_auto)
-            self.h1_edit.returnPressed.connect(self.change_heater)
-            self.h2_checkbox.clicked.connect(self.change_heater_auto)
+            # enable controls
+            self.t2_edit.setEnabled(True)
+            self.r1_edit.setEnabled(True)
+            self.r2_checkbox.setEnabled(True)
+            self.gf1_edit.setEnabled(True)
+            self.gf2_checkbox.setEnabled(True)
+            self.gf3_edit.setEnabled(True)
+            self.h1_edit.setEnabled(True)
+            self.h2_checkbox.setEnabled(True)
 
         elif not connected:
             self.display_error('Connection lost.')
@@ -196,14 +231,17 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
             self.modulesAction.setEnabled(False)
             self.readingsAction.setEnabled(False)
 
-            # disconnect user input from mercury
-            self.t2_edit.returnPressed.disconnect(self.change_t_setpoint)
-            self.r1_edit.returnPressed.disconnect(self.change_ramp)
-            self.r2_checkbox.clicked.disconnect(self.change_ramp_auto)
-            self.gf1_edit.returnPressed.disconnect(self.change_flow)
-            self.gf2_checkbox.clicked.disconnect(self.change_flow_auto)
-            self.h1_edit.returnPressed.disconnect(self.change_heater)
-            self.h2_checkbox.clicked.disconnect(self.change_heater_auto)
+            # disable controls
+            self.t2_edit.setEnabled(False)
+            self.r1_edit.setEnabled(False)
+            self.r2_checkbox.setEnabled(False)
+            self.gf1_edit.setEnabled(False)
+            self.gf2_checkbox.setEnabled(False)
+            self.gf3_edit.setEnabled(False)
+            self.h1_edit.setEnabled(False)
+            self.h2_checkbox.setEnabled(False)
+
+        self.build_sensor_menu()
 
     def display_message(self, text):
         self.statusbar.showMessage('%s' % text, 5000)
@@ -217,6 +255,8 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         Parses readings for the MercuryMonitorApp and updates UI accordingly
         """
 
+        self.labelTitle.setText(self.TITLE_TEMPLATE + ': ' + self.feed.temperature_module_nick)
+
         # heater signals
         self.h1_label.setText('Heater, %s V:' % readings['HeaterVolt'])
         self.h1_edit.updateValue(readings['HeaterPercent'])
@@ -234,7 +274,7 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
 
         # gas flow signals
         self.gf1_edit.updateValue(readings['FlowPercent'])
-        self.gf1_label.setText('Gas flow (min = %s%%):' % readings['FlowMin'])
+        self.gf3_edit.updateValue(readings['FlowMin'])
 
         if self.feed.gasflow:
             is_gf_auto = readings['FlowAuto'] == 'ON'
@@ -242,10 +282,11 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
             self.gf1_edit.setEnabled(not is_gf_auto)
             self.gf2_checkbox.setChecked(is_gf_auto)
             self.gf2_checkbox.setEnabled(True)
+            self.gf3_edit.setEnabled(True)
         else:
-            self.gf1_edit.setEnabled(True)
             self.gf1_edit.setEnabled(False)
             self.gf2_checkbox.setEnabled(False)
+            self.gf3_edit.setEnabled(False)
 
         # temperature signals
         self.t1_reading.setText('%s K' % round(readings['Temp'], 3))
@@ -255,6 +296,18 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         is_ramp_enable = readings['TempRampEnable'] == 'ON'
         self.r2_checkbox.setChecked(is_ramp_enable)
 
+        # alarms
+        alarm_str = ''
+        for k, v in readings['Alarms'].items():
+            alarm_str += '{}: {} '.format(k, v)
+
+        self.alarm_label.setText(alarm_str)
+
+        if alarm_str:
+            self.alarm_label.show()
+        else:
+            self.alarm_label.hide()
+
     @QtCore.pyqtSlot(object)
     def update_plot(self, readings):
         # append data for plotting
@@ -276,6 +329,18 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         self.canvas.update_data(self.xdata_min_zero, self.ydata_tmpr,
                                 self.ydata_gflw, self.ydata_htr)
 
+    def clear_plot(self):
+        # append data for plotting
+        self.xdata = np.array([])
+        self.xdata_min_zero = np.array([])
+        self.ydata_tmpr = np.array([])
+        self.ydata_gflw = np.array([])
+        self.ydata_htr = np.array([])
+
+        # update plot
+        self.canvas.update_data(self.xdata, self.ydata_tmpr,
+                                self.ydata_gflw, self.ydata_htr)
+
 # =================== LOGGING DATA ============================================
 
     def setup_logging(self):
@@ -338,7 +403,7 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
 
     def log_temperature_data(self):
         # save temperature data to log file
-        if self.mercury.connected:
+        if self.feed.connected:
             self.save_temperature_data(self.log_file)
 
 # =================== CALLBACKS FOR SETTING CHANGES ===========================
@@ -371,7 +436,12 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
     @QtCore.pyqtSlot()
     def change_flow(self):
         self.feed.temperature.loop_fset = self.gf1_edit.value()
-        self.display_message('Gas flow  = %s%%' % self.gf1_edit.value())
+        self.display_message('Gas flow = %s%%' % self.gf1_edit.value())
+
+    @QtCore.pyqtSlot()
+    def change_flow_min(self):
+        self.feed.gasflow.gmin = self.gf3_edit.value()
+        self.display_message('Gas flow min = %s%%' % self.gf3_edit.value())
 
     @QtCore.pyqtSlot(bool)
     def change_flow_auto(self, checked):
@@ -406,6 +476,18 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
 
 # ========================== CALLBACKS FOR MENU BAR ===========================
 
+    def on_sensor_selected(self, action):
+
+        nick = action.data()
+
+        try:
+            self.feed.select_temp_sensor(nick)
+        except IOError:
+            # the selected module name does not exists -> rebuild our list
+            self.build_sensor_menu()
+        else:
+            self.clear_plot()
+
     @QtCore.pyqtSlot()
     def on_readings_clicked(self):
         # create readings overview window if not present
@@ -418,8 +500,9 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
     def on_module_selection_clicked(self):
         # create readings overview window if not present
         if self.modulesDialog is None:
-            self.modulesDialog = ModulesDialog(self.feed)
+            self.modulesDialog = ModulesDialog(self.mercury)
         # show it
+        self.modulesDialog.update_gui()
         self.modulesDialog.open()
 
     @QtCore.pyqtSlot()
@@ -435,6 +518,12 @@ class MercuryMonitorApp(QtWidgets.QMainWindow):
         else:
             subprocess.Popen(['xdg-open', self.logging_path])
 
+    def _get_nicks(self, sensor_type):
+        if self.feed.connected:
+            return list(m.nick for m in self.mercury.modules if type(m) == sensor_type)
+        else:
+            return []
+
 
 # noinspection PyUnresolvedReferences
 class ReadingsTab(QtWidgets.QWidget):
@@ -489,13 +578,8 @@ class ReadingsTab(QtWidgets.QWidget):
         """Gets alarms of associated module."""
 
         # get alarms for all modules
-        address = self.module.address.split(':')
-        short_address = address[1]
-        if self.module.nick == 'LOOP':
-            short_address = short_address.split('.')
-            short_address = short_address[0] + '.loop1'
         try:
-            alarm = self.mercury.alarms[short_address]
+            alarm = self.mercury.alarms[self.module.uid]
         except KeyError:
             alarm = '--'
 
@@ -506,6 +590,7 @@ class ReadingsOverview(QtWidgets.QDialog):
 
     def __init__(self, mercury, parent=None):
         super(self.__class__, self).__init__(parent=parent)
+        self.setWindowTitle('Readings Overview')
         self.mercury = mercury
         self.setupUi(self)
 
@@ -547,6 +632,10 @@ class ReadingsOverview(QtWidgets.QDialog):
             self.tabWidget.currentWidget().get_alarms()
 
 
+class _NoModule:
+    nick = 'None'
+
+
 class ModulesDialog(QtWidgets.QDialog):
     """
     Provides a user dialog to select the modules for the feed.
@@ -554,50 +643,63 @@ class ModulesDialog(QtWidgets.QDialog):
 
     accepted = QtCore.pyqtSignal(object)
 
-    def __init__(self, mercury_feed, parent=None):
+    def __init__(self, mercury, parent=None):
         super(self.__class__, self).__init__(parent=parent)
         uic.loadUi(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                 'module_dialog.ui'), self)
 
-        self.feed = mercury_feed
-        self.modules = self.feed.mercury.modules
+        self.mercury = mercury
+        self.update_gui()
 
-        def get_nicks(type_):
-            return list(m.nick for m in self.modules if type(m) == type_)
+    def update_gui(self):
 
-        self.temp_module_nicks = get_nicks(MercuryITC_TEMP)
-        self.htr_module_nicks = get_nicks(MercuryITC_HTR)
-        self.aux_module_nicks = get_nicks(MercuryITC_AUX)
+        self.temp_modules = self._get_modules_for_type(MercuryITC_TEMP)
+        self.htr_modules = self._get_modules_for_type(MercuryITC_HTR)
+        self.aux_modules = self._get_modules_for_type(MercuryITC_AUX)
 
-        self.htr_module_nicks.append('None')
-        self.aux_module_nicks.append('None')
+        self.htr_modules.append(_NoModule())
+        self.aux_modules.append(_NoModule())
 
-        self.comboBoxTEMP.addItems(self.temp_module_nicks)
-        self.comboBoxHTR.addItems(self.htr_module_nicks)
-        self.comboBoxAUX.addItems(self.aux_module_nicks)
+        self.comboBoxTEMP.addItems([m.nick for m in self.temp_modules])
+        self.comboBoxHTR.addItems([m.nick for m in self.htr_modules])
+        self.comboBoxAUX.addItems([m.nick for m in self.aux_modules])
 
         # get current modules
-        self.comboBoxTEMP.setCurrentText(self.feed.temperature.nick)
-        self.comboBoxHTR.setCurrentText(self.feed.temperature.loop_htr)
-        self.comboBoxAUX.setCurrentText(self.feed.temperature.loop_aux)
+        self.comboBoxTEMP.setCurrentIndex(0)
+        self.comboBoxHTR.setCurrentText(self.temp_modules[0].loop_htr)
+        self.comboBoxAUX.setCurrentText(self.temp_modules[0].loop_aux)
 
         # connect callbacks
-        self.comboBoxTEMP.currentTextChanged.connect(self._on_comboBoxTEMP_textChanged)
-        self.buttonBox.accepted.connect(self._on_accept)
+        self.comboBoxTEMP.currentIndexChanged.connect(self.on_temp_selected)
+        self.buttonBox.accepted.connect(self.on_accept)
 
-    @QtCore.pyqtSlot(str)
-    def _on_comboBoxTEMP_textChanged(self, text):
+    @QtCore.pyqtSlot(int)
+    def on_temp_selected(self, index):
         # update content of heater and gasflow combo boxes
-        temp_module = next(m for m in self.modules if m.nick == text)
-
-        self.comboBoxHTR.setCurrentText(temp_module.loop_htr)
-        self.comboBoxAUX.setCurrentText(temp_module.loop_aux)
+        self.comboBoxHTR.setCurrentText(self.temp_modules[index].loop_htr)
+        self.comboBoxAUX.setCurrentText(self.temp_modules[index].loop_aux)
 
     @QtCore.pyqtSlot()
-    def _on_accept(self):
-        temp_nick = self.comboBoxTEMP.currentText()
-        self.feed.select_temp_sensor(temp_nick)  # updated feed
-        CONF.set('MercuryFeed', 'temperature_module', temp_nick)  # write to config file
+    def on_accept(self):
+        temp_index = self.comboBoxTEMP.currentIndex()
+        htr_nick = self.comboBoxHTR.currentText()
+        aux_nick = self.comboBoxAUX.currentText()
+
+        # remove heater and gasflow modules from previous loop
+        for module in self.temp_modules:
+            if module is not self.temp_modules[temp_index]:
+                if module.loop_htr == htr_nick:
+                    module.loop_htr = 'None'
+                if module.loop_aux == aux_nick:
+                    module.loop_aux = 'None'
+
+        # assign heater and gasflow modules to selected loop
+        self.temp_modules[temp_index].loop_htr = htr_nick
+        self.temp_modules[temp_index].loop_aux = aux_nick
+
+    def _get_modules_for_type(self, sensor_type):
+
+        return [m for m in self.mercury.modules if type(m) is sensor_type]
 
 
 def run():
